"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connection = void 0;
const querystring_1 = __importDefault(require("querystring"));
const axios_1 = __importDefault(require("axios"));
const authUrl = 'https://sso.ci.ford.com/';
const applicationId = '71A3AD0A-CF46-4CCF-B473-FC7FE5BC4592';
const clientId = '9fb503e0-715b-47e8-adfd-ad4b7770f73b';
const userAgent = 'FordPass/5 CFNetwork/1325.0.1 Darwin/21.1.0';
class Connection {
    constructor(config, log) {
        this.config = config;
        this.log = log;
    }
    auth() {
        return __awaiter(this, void 0, void 0, function* () {
            const url = authUrl + 'v1.0/endpoint/default/token';
            const options = {
                method: 'POST',
                url: url,
                headers: {
                    'Content-Type': 'application/x-www-form-urlencoded',
                    'User-Agent': userAgent,
                },
                data: querystring_1.default.stringify({
                    client_id: clientId,
                    grant_type: 'password',
                    username: this.config.username,
                    password: this.config.password,
                }),
            };
            try {
                const result = yield (0, axios_1.default)(options);
                if (result.status === 200 && result.data.access_token) {
                    const nextResult = yield axios_1.default.put('https://api.mps.ford.com/api/oauth2/v1/token', {
                        code: result.data.access_token,
                    }, {
                        headers: {
                            'Content-Type': 'application/json',
                            'User-Agent': userAgent,
                            'Application-Id': applicationId,
                        },
                    });
                    if (nextResult.status === 200 && nextResult.data.access_token) {
                        this.config.access_token = nextResult.data.access_token;
                        this.config.refresh_token = nextResult.data.refresh_token;
                        return nextResult.data;
                    }
                    else {
                        this.log.error(`Auth failed with status: ${nextResult.status}`);
                    }
                }
                else {
                    this.log.error(`Auth failed with status: ${result.status}`);
                }
                return;
            }
            catch (error) {
                this.log.error(`Auth failed with error: ${error.code || error.response.status}`);
                return;
            }
        });
    }
    refreshAuth() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.config.refresh_token) {
                    const result = yield axios_1.default.put('https://api.mps.ford.com/api/oauth2/v1/refresh', {
                        refresh_token: this.config.refresh_token,
                    }, {
                        headers: {
                            'Content-Type': 'application/json',
                            'User-Agent': userAgent,
                            'Application-Id': applicationId,
                        },
                    });
                    if (result.status === 200 && result.data.access_token) {
                        this.config.access_token = result.data.access_token;
                        this.config.refresh_token = result.data.refresh_token;
                        return result.data;
                    }
                    else {
                        this.log.error(`Auth failed with status: ${result.status}`);
                    }
                }
                else {
                    return yield this.auth();
                }
                return;
            }
            catch (error) {
                this.log.error(`Auth failed with error: ${error.code || error.response.status}`);
                return;
            }
        });
    }
}
exports.Connection = Connection;
//# sourceMappingURL=fordpass-connection.js.map