"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const fordpass_1 = require("./fordpass");
const vehicle_1 = require("./types/vehicle");
const fordpass_connection_1 = require("./fordpass-connection");
const accessory_1 = require("./accessory");
let hap;
let Accessory;
const PLUGIN_NAME = 'homebridge-fordpass';
const PLATFORM_NAME = 'FordPass';
class FordPassPlatform {
    constructor(log, config, api) {
        this.accessories = [];
        this.vehicles = [];
        this.pendingLockUpdate = false;
        this.log = log;
        this.api = api;
        this.config = config;
        if (!config) {
            return;
        }
        if (!config.username || !config.password || !config.vehicles) {
            this.log.error('Please add a userame, password, and vehicles to your config.json');
            return;
        }
        api.on("didFinishLaunching", this.didFinishLaunching.bind(this));
    }
    configureAccessory(accessory) {
        var _a;
        const self = this;
        this.log.info(`Configuring accessory ${accessory.displayName}`);
        accessory.on("identify", () => {
            this.log.info(`${accessory.displayName} identified!`);
        });
        const vehicle = new fordpass_1.Vehicle(accessory.context.name, accessory.context.vin, this.config, this.log);
        const fordAccessory = new accessory_1.FordpassAccessory(accessory);
        const defaultState = hap.Characteristic.LockTargetState.UNSECURED;
        const lockService = fordAccessory.createService(hap.Service.LockMechanism);
        const switchService = fordAccessory.createService(hap.Service.Switch);
        const batteryService = fordAccessory.createService(hap.Service.Battery, ((_a = this.config.options) === null || _a === void 0 ? void 0 : _a.batteryName) || 'Fuel Level');
        lockService.setCharacteristic(hap.Characteristic.LockCurrentState, defaultState);
        lockService
            .setCharacteristic(hap.Characteristic.LockTargetState, defaultState)
            .getCharacteristic(hap.Characteristic.LockTargetState)
            .on("set", (value, callback) => __awaiter(this, void 0, void 0, function* () {
            this.log.debug(`${value ? 'Locking' : 'Unlocking'} ${accessory.displayName}`);
            let commandId = '';
            let command = vehicle_1.Command.LOCK;
            if (value === hap.Characteristic.LockTargetState.UNSECURED) {
                command = vehicle_1.Command.UNLOCK;
            }
            commandId = yield vehicle.issueCommand(command);
            let tries = 30;
            this.pendingLockUpdate = true;
            const self = this;
            const interval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                if (tries > 0) {
                    const status = yield vehicle.commandStatus(command, commandId);
                    if ((status === null || status === void 0 ? void 0 : status.status) === 200) {
                        lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, value);
                        self.pendingLockUpdate = false;
                        clearInterval(interval);
                    }
                    tries--;
                }
                else {
                    self.pendingLockUpdate = false;
                    clearInterval(interval);
                }
            }), 1000);
            callback(undefined, value);
        }))
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _b;
            let lockNumber = hap.Characteristic.LockTargetState.UNSECURED;
            const lockStatus = (_b = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _b === void 0 ? void 0 : _b.lockStatus.value;
            if (lockStatus === 'LOCKED') {
                lockNumber = hap.Characteristic.LockTargetState.SECURED;
            }
            callback(undefined, lockNumber);
            const status = yield vehicle.status();
            if (status) {
                let lockNumber = hap.Characteristic.LockTargetState.UNSECURED;
                const lockStatus = status.lockStatus.value;
                if (lockStatus === 'LOCKED') {
                    lockNumber = hap.Characteristic.LockTargetState.SECURED;
                }
                lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, lockNumber);
                lockService.updateCharacteristic(hap.Characteristic.LockTargetState, lockNumber);
            }
            else {
                self.log.error(`Cannot get information for ${accessory.displayName} lock`);
            }
        }));
        switchService
            .setCharacteristic(hap.Characteristic.On, false)
            .getCharacteristic(hap.Characteristic.On)
            .on("set", (value, callback) => __awaiter(this, void 0, void 0, function* () {
            this.log.debug(`${value ? 'Starting' : 'Stopping'} ${accessory.displayName}`);
            if (value) {
                yield vehicle.issueCommand(vehicle_1.Command.START);
            }
            else {
                yield vehicle.issueCommand(vehicle_1.Command.STOP);
            }
            callback(undefined, value);
        }))
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _c;
            const engineStatus = ((_c = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _c === void 0 ? void 0 : _c.remoteStartStatus.value) || 0;
            callback(undefined, engineStatus);
            const status = yield vehicle.status();
            if (status) {
                let started = false;
                const engineStatus = status.remoteStartStatus.value || 0;
                if (engineStatus > 0) {
                    started = true;
                }
                switchService.updateCharacteristic(hap.Characteristic.On, started);
            }
            else {
                self.log.error(`Cannot get information for ${accessory.displayName} engine`);
            }
        }));
        batteryService
            .setCharacteristic(hap.Characteristic.BatteryLevel, 100)
            .getCharacteristic(hap.Characteristic.BatteryLevel)
            .on("get", (callback) => __awaiter(this, void 0, void 0, function* () {
            var _d, _e, _f, _g, _h, _j, _k, _l;
            const fuel = (_e = (_d = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _d === void 0 ? void 0 : _d.fuel) === null || _e === void 0 ? void 0 : _e.fuelLevel;
            const battery = (_g = (_f = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _f === void 0 ? void 0 : _f.batteryFillLevel) === null || _g === void 0 ? void 0 : _g.value;
            let level = fuel || battery || 100;
            if (level > 100) {
                level = 100;
            }
            if (level < 0) {
                level = 0;
            }
            callback(undefined, level);
            const status = yield vehicle.status();
            if (status) {
                const fuel = (_h = status.fuel) === null || _h === void 0 ? void 0 : _h.fuelLevel;
                const battery = (_j = status.batteryFillLevel) === null || _j === void 0 ? void 0 : _j.value;
                const chargingStatus = (_l = (_k = vehicle === null || vehicle === void 0 ? void 0 : vehicle.info) === null || _k === void 0 ? void 0 : _k.chargingStatus) === null || _l === void 0 ? void 0 : _l.value;
                let level = fuel || battery || 100;
                if (level > 100) {
                    level = 100;
                }
                if (level < 0) {
                    level = 0;
                }
                batteryService.updateCharacteristic(hap.Characteristic.BatteryLevel, level);
                if (battery) {
                    if (chargingStatus === 'ChargingAC') {
                        batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.CHARGING);
                    }
                    else {
                        batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.NOT_CHARGING);
                    }
                }
                else {
                    batteryService.updateCharacteristic(hap.Characteristic.ChargingState, hap.Characteristic.ChargingState.NOT_CHARGEABLE);
                }
                if (level < 10) {
                    batteryService.updateCharacteristic(hap.Characteristic.StatusLowBattery, hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
                }
                else {
                    batteryService.updateCharacteristic(hap.Characteristic.StatusLowBattery, hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                }
            }
            else {
                self.log.error(`Cannot get information for ${accessory.displayName} engine`);
            }
        }));
        this.vehicles.push(vehicle);
        this.accessories.push(accessory);
    }
    didFinishLaunching() {
        return __awaiter(this, void 0, void 0, function* () {
            const self = this;
            const ford = new fordpass_connection_1.Connection(this.config, this.log);
            const authInfo = yield ford.auth();
            if (authInfo) {
                setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    self.log.debug('Reauthenticating with refresh token');
                    yield ford.refreshAuth();
                }), authInfo.expires_in * 1000 - 10000);
                yield this.addVehicles();
                yield this.updateVehicles();
                yield this.refreshVehicles();
                setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    yield self.updateVehicles();
                }), 60 * 1000);
            }
        });
    }
    addVehicles() {
        return __awaiter(this, void 0, void 0, function* () {
            const vehicles = this.config.vehicles;
            vehicles === null || vehicles === void 0 ? void 0 : vehicles.forEach((vehicle) => __awaiter(this, void 0, void 0, function* () {
                vehicle.vin = vehicle.vin.toUpperCase();
                const uuid = hap.uuid.generate(vehicle.vin);
                const accessory = new Accessory(vehicle.name, uuid);
                accessory.context.name = vehicle.name;
                accessory.context.vin = vehicle.vin;
                const accessoryInformation = accessory.getService(hap.Service.AccessoryInformation);
                if (accessoryInformation) {
                    accessoryInformation.setCharacteristic(hap.Characteristic.Manufacturer, 'Ford');
                    accessoryInformation.setCharacteristic(hap.Characteristic.Model, vehicle.name);
                    accessoryInformation.setCharacteristic(hap.Characteristic.SerialNumber, vehicle.vin);
                }
                if (!this.accessories.find((x) => x.UUID === uuid)) {
                    this.log.debug(`New vehicle found: ${vehicle.name}`);
                    this.configureAccessory(accessory);
                    this.api.registerPlatformAccessories(PLUGIN_NAME, PLATFORM_NAME, [accessory]);
                }
            }));
            this.accessories.forEach((accessory) => {
                if (!(vehicles === null || vehicles === void 0 ? void 0 : vehicles.find((x) => x.vin === accessory.context.vin))) {
                    this.api.unregisterPlatformAccessories(PLUGIN_NAME, PLATFORM_NAME, [accessory]);
                    const index = this.accessories.indexOf(accessory);
                    if (index > -1) {
                        this.accessories.splice(index, 1);
                        this.vehicles.slice(index, 1);
                    }
                }
            });
        });
    }
    updateVehicles() {
        return __awaiter(this, void 0, void 0, function* () {
            this.vehicles.forEach((vehicle) => __awaiter(this, void 0, void 0, function* () {
                const status = yield vehicle.status();
                this.log.debug(`Updating info for ${vehicle.name}`);
                const lockStatus = status === null || status === void 0 ? void 0 : status.lockStatus.value;
                let lockNumber = hap.Characteristic.LockCurrentState.UNSECURED;
                if (lockStatus === 'LOCKED') {
                    lockNumber = hap.Characteristic.LockCurrentState.SECURED;
                }
                const engineStatus = (status === null || status === void 0 ? void 0 : status.remoteStartStatus.value) || 0;
                let started = false;
                if (engineStatus > 0) {
                    started = true;
                }
                const uuid = hap.uuid.generate(vehicle.vin);
                const accessory = this.accessories.find((x) => x.UUID === uuid);
                if (!this.pendingLockUpdate) {
                    const lockService = accessory === null || accessory === void 0 ? void 0 : accessory.getService(hap.Service.LockMechanism);
                    lockService && lockService.updateCharacteristic(hap.Characteristic.LockCurrentState, lockNumber);
                    lockService && lockService.updateCharacteristic(hap.Characteristic.LockTargetState, lockNumber);
                }
                const switchService = accessory === null || accessory === void 0 ? void 0 : accessory.getService(hap.Service.Switch);
                switchService && switchService.updateCharacteristic(hap.Characteristic.On, started);
            }));
        });
    }
    refreshVehicles() {
        return __awaiter(this, void 0, void 0, function* () {
            this.vehicles.forEach((vehicle) => __awaiter(this, void 0, void 0, function* () {
                if (vehicle.autoRefresh && vehicle.refreshRate && vehicle.refreshRate > 0) {
                    this.log.debug(`Configuring ${vehicle.name} to refresh every ${vehicle.refreshRate} minutes.`);
                    setInterval(() => __awaiter(this, void 0, void 0, function* () {
                        this.log.debug(`Refreshing info for ${vehicle.name}`);
                        yield vehicle.issueCommand(vehicle_1.Command.REFRESH);
                    }), 60000 * vehicle.refreshRate);
                }
            }));
        });
    }
}
module.exports = (api) => {
    hap = api.hap;
    Accessory = api.platformAccessory;
    api.registerPlatform(PLUGIN_NAME, PLATFORM_NAME, FordPassPlatform);
};
//# sourceMappingURL=index.js.map